package com.liangjian.dataplatform.member.service.impl;

import com.liangjian.dataplatform.member.dao.secondary.UserLevelDescRepository;
import com.liangjian.dataplatform.member.dao.secondary.UserLevelUnitRepository;
import com.liangjian.dataplatform.member.entity.po.secondary.UserLevelDesc;
import com.liangjian.dataplatform.member.entity.po.secondary.UserLevelUnit;
import com.liangjian.dataplatform.member.util.StringUtil;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.jpa.domain.Specification;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;
import com.liangjian.dataplatform.member.service.IUserLevelService;

import javax.persistence.criteria.Predicate;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.*;
import java.util.stream.Collectors;

/**
 * @Author: 黄子奇
 * @Date: 2019/11/15
 */
@Service
public class UserLevelService implements IUserLevelService {
    @Autowired
    private UserLevelDescRepository userLevelDescRepository;
    @Autowired
    private UserLevelUnitRepository userLevelUnitRepository;

    @Override
    public UserLevelDesc getLevelDesc() {
        List<UserLevelDesc> all = userLevelDescRepository.findAll();
        if (all.isEmpty()) {
            return new UserLevelDesc();
        } else {
            return all.get(0);
        }
    }

    @Override
    @Transactional(readOnly = false, propagation = Propagation.REQUIRED, rollbackFor = Exception.class)
    public boolean setLevelDesc(UserLevelDesc userLevelDesc) {
        List<UserLevelDesc> all = userLevelDescRepository.findAll();
        try {
            if (all.isEmpty()) {
                userLevelDescRepository.saveWithCycle(userLevelDesc);
            } else {
                userLevelDesc.setId(all.get(0).getId());
                userLevelDescRepository.saveWithCycle(userLevelDesc);
            }
            return true;
        } catch (Exception ex) {
            ex.printStackTrace();
            return false;
        }
    }

    @Override
    public List<UserLevelUnit> getAllLevelUnit() {
        return userLevelUnitRepository.findAll();
    }

    @Override
    @Transactional(readOnly = false, propagation = Propagation.REQUIRED, rollbackFor = Exception.class)
    public boolean addUserLevelUnit(UserLevelUnit userLevelUnit) {
        if (userLevelUnit.getId()!=null)
            return false;
        try {
            userLevelUnitRepository.saveWithCycle(userLevelUnit);
            return true;
        } catch (Exception ex) {
            ex.printStackTrace();
            return false;
        }
    }

    @Override
    public Optional<UserLevelUnit> getById(String id) {
        return userLevelUnitRepository.findById(id);
    }

    @Override
    @Transactional(readOnly = false, propagation = Propagation.REQUIRED, rollbackFor = Exception.class)
    public boolean saveUserLevelUnit(UserLevelUnit userLevelUnit) {
        if (userLevelUnit.getId()==null || userLevelUnit.getId().equals(0))
            return false;
        try {
            userLevelUnitRepository.saveWithCycle(userLevelUnit);
            return true;
        } catch (Exception ex) {
            ex.printStackTrace();
            return false;
        }
    }

    @Override
    public boolean deleteUserLevelUnit(String id) {
        try {
            userLevelUnitRepository.deleteById(id);
            return true;
        } catch (Exception ex) {
            ex.printStackTrace();
            return false;
        }
    }

    @Override
    public Optional<UserLevelUnit> getLevelIn(Integer balance) {
        Specification<UserLevelUnit> specification = (Specification<UserLevelUnit>) (root, criteriaQuery, criteriaBuilder) -> {
            Predicate p1 = criteriaBuilder.le(root.get("growthFrom"), balance);
            Predicate p2 = criteriaBuilder.ge(root.get("growthTo"), balance);
            return criteriaBuilder.and(p1, p2);
        };

        var res = userLevelUnitRepository.findOne(specification);
        if (res.isEmpty()) {
            var units = userLevelUnitRepository.findAll();
            for (UserLevelUnit unit : units) {
                if (balance > unit.getGrowthTo() && balance > unit.getGrowthFrom()) {
                    res = Optional.of(unit);
                }
            }
        }

        return res;
    }

    @Override
    @Transactional(readOnly = false, propagation = Propagation.REQUIRED, rollbackFor = Exception.class)
    public boolean saveAll(HashMap hash) throws ParseException {
        ArrayList<UserLevelUnit> UserLevelUnitList = new ArrayList<>();
        ArrayList<HashMap<String, Object>> arrayList = (ArrayList<HashMap<String, Object>>) hash.get("list");
        for (HashMap<String, Object> objectArrayList : arrayList) {
            UserLevelUnit userLevelUnit = new UserLevelUnit();
            userLevelUnit.setName((String) objectArrayList.get("name"));
            userLevelUnit.setId(Long.valueOf(objectArrayList.get("id").toString()));
            userLevelUnit.setLevel((Integer) objectArrayList.get("level"));
            userLevelUnit.setGrowthFrom((Integer) objectArrayList.get("growthFrom"));
            userLevelUnit.setGrowthTo((Integer) objectArrayList.get("growthTo"));
            UserLevelUnitList.add(userLevelUnit);
        }
        HashMap<String, Object> map = (HashMap<String, Object>) hash.get("desc");
        UserLevelDesc userLevelDesc = new UserLevelDesc();
        SimpleDateFormat sdf=new SimpleDateFormat("yyyy-MM-dd");
        String crtDateString = (String) map.get("crtDate");
        Date crtDate = crtDateString==null?new Date():sdf.parse(crtDateString);
        userLevelDesc.setCrtDate(crtDate);
        userLevelDesc.setCrtUser((String)map.get("crtUser"));
        userLevelDesc.setLastUpdate(new Date());
        userLevelDesc.setId(Long.valueOf(map.get("id").toString()));
        userLevelDesc.setDescription((String)map.get("description"));
        //找出原本所有的
//        List<String> upIds = levelListAndDesc.getList().stream().map(i -> i.getId()).collect(Collectors.toList());
//        List<UserLevelUnit> all = getAllLevelUnit();
//        List<UserLevelUnit> toDelete = all.stream().filter(i -> !upIds.contains(i.getId())).collect(Collectors.toList());

//        for (UserLevelUnit userLevelUnit : UserLevelUnitList) {
//            if (!StringUtil.isEmpty(userLevelUnit.getName())) {
//                userLevelUnitRepository.delete(userLevelUnit);
//            }
//        }
        try {
            userLevelUnitRepository.saveAllWithCycle(UserLevelUnitList);
            userLevelDescRepository.saveWithCycle(userLevelDesc);
//            userLevelUnitRepository.save(objects);
            return true;
        } catch (Exception ex) {
            ex.printStackTrace();
            return false;
        }
    }
}
